package co.hyperverge.hyperkyc_flutter;

import static co.hyperverge.hyperkyc_flutter.FLHyperKycConstants.TRANSACTION_ID;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;

import androidx.annotation.NonNull;

import java.util.HashMap;
import java.util.Map;

import co.hyperverge.hyperkyc.HyperKyc;
import co.hyperverge.hyperkyc.data.models.HyperKycConfig;
import co.hyperverge.hyperkyc.data.models.result.HyperKycResult;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;

/**
 * HyperkycFlutterPlugin
 */
public class HyperkycFlutterPlugin implements FlutterPlugin, ActivityAware, PluginRegistry.ActivityResultListener {
    private static final String TAG = HyperkycFlutterPlugin.class.getCanonicalName();
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel hyperKycChannel;
    protected Context context;
    protected Activity activity;
    private Result pendingResult;
    private final FLHyperKyc flHyperKyc = new FLHyperKyc();
    private final HyperKyc.Contract contract = new HyperKyc.Contract();
    private HyperKycConfig hyperKycConfig;
    private String transactionId;

    private final MethodCallHandler hyperKycMethodCallHandler = (@NonNull MethodCall call, @NonNull Result result) -> {
        pendingResult = result;
        if (call.method.equalsIgnoreCase(FLHyperKycConstants.CREATE_UNIQUE_ID)) {
            String uniqueId = flHyperKyc.createUniqueId();
            result.success(uniqueId);
        } else if (call.method.equalsIgnoreCase(FLHyperKycConstants.LAUNCH)) {
            HashMap<String, Object> hyperKycConfigMap = call.argument(FLHyperKycConstants.HYPERKYCCONFIG);
            assert hyperKycConfigMap != null;
            if (hyperKycConfigMap.containsKey(TRANSACTION_ID)) {
                transactionId = (String) hyperKycConfigMap.get(TRANSACTION_ID);
            }
            hyperKycConfig = flHyperKyc.getHyperKycConfigFromMap(hyperKycConfigMap);
            Intent contractIntent = contract.createIntent(activity, hyperKycConfig);
            activity.startActivityForResult(contractIntent, FLHyperKycConstants.REQUEST_CODE_HYPERKYC_LAUNCH);
        } else {
            result.notImplemented();
        }
    };

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        this.context = flutterPluginBinding.getApplicationContext();
        hyperKycChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), FLHyperKycConstants.HYPERKYC_CHANNEL_NAME);
        hyperKycChannel.setMethodCallHandler(hyperKycMethodCallHandler);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        hyperKycChannel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.activity = binding.getActivity();
        binding.addActivityResultListener(this);
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        // Use this function only when necessary
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        // Use this function only when necessary
    }

    @Override
    public void onDetachedFromActivity() {
        // Use this function only when necessary
    }

    @Override
    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case FLHyperKycConstants.REQUEST_CODE_HYPERKYC_LAUNCH:
                if (data != null) {
                    HyperKycResult hyperKycResult = contract.parseResult(resultCode, data);
                    if (transactionId != null && hyperKycResult.getTransactionId().equalsIgnoreCase(transactionId)) {
                        handleHyperKycResultData(hyperKycResult);
                    }
                }
                break;
            default:
                return false;
        }
        return true;
    }

    private void handleHyperKycResultData(HyperKycResult hyperKycResult) {
        Map<String, Object> resultMap = processHyperKycResult(hyperKycResult);
        pendingResult.success(resultMap);
    }

    private Map<String, Object> processHyperKycResult(HyperKycResult hyperKycResult) {
        Map<String, Object> resultMap = new HashMap<>();

        String status, transactionId, errorMessage = null, latestModule = null, rawDataJsonString = null;
        Map<String, String> details;
        int errorCode = -1000;
        status = hyperKycResult.getStatus();
        transactionId = hyperKycResult.getTransactionId();
        details = hyperKycResult.getDetails();

        if (hyperKycResult.getErrorCode() != null) {
            errorCode = hyperKycResult.getErrorCode();
        }
        if (hyperKycResult.getErrorMessage() != null) {
            errorMessage = hyperKycResult.getErrorMessage();
        }
        if (hyperKycResult.getLatestModule() != null) {
            latestModule = hyperKycResult.getLatestModule();
        }
        if (hyperKycResult.getRawDataJsonString() != null) {
            rawDataJsonString = hyperKycResult.getRawDataJsonString();
        }

        resultMap.put(FLHyperKycConstants.STATUS, status);
        resultMap.put(TRANSACTION_ID, transactionId);
        resultMap.put(FLHyperKycConstants.DETAILS, details);
        resultMap.put(FLHyperKycConstants.ERROR_CODE, errorCode);
        resultMap.put(FLHyperKycConstants.ERROR_MESSAGE, errorMessage);
        resultMap.put(FLHyperKycConstants.LATEST_MODULE, latestModule);
        resultMap.put(FLHyperKycConstants.RAW_DATA_JSON_STRING, rawDataJsonString);

        return resultMap;
    }
}