//
//  FLHyperKyc.swift
//  hyperkyc_flutter
//
//  Created by Shrirama Upadhya A on 22/01/22.
//

import Foundation
import HyperKYC

class FLHyperKyc {
    
    private static let APP_ID = "appId"
    private static let APP_KEY = "appKey"
    private static let ACCESS_TOKEN = "accessToken"
    private static let WORKFLOW_ID = "workflowId"
    private static let TRANSACTION_ID = "transactionId"
    private static let INPUTS = "inputs"
    private static let DEFAULT_LANG_CODE = "defaultLangCode"
    private static let USE_LOCATION = "useLocation"
    private static let UNIQUE_ID = "uniqueId"
    private static let METADATA = "metadata"
    
    public static func createUniqueId() -> String {
        return HyperKyc.createUniqueId()
    }
    
    public static func launch(context: UIViewController, hyperKycConfigArgument: [String: Any], callback: @escaping ((HyperKycResult) -> Void)) {
        let appId = hyperKycConfigArgument[APP_ID] as? String
        let appKey = hyperKycConfigArgument[APP_KEY] as? String
        let accessToken = hyperKycConfigArgument[ACCESS_TOKEN] as? String
        let workflowId = hyperKycConfigArgument[WORKFLOW_ID] as! String
        let transactionId = hyperKycConfigArgument[TRANSACTION_ID] as! String
        let inputs = hyperKycConfigArgument[INPUTS] as? [String: String]
        let defaultLangCode = hyperKycConfigArgument[DEFAULT_LANG_CODE] as? String
        let useLocation = hyperKycConfigArgument[USE_LOCATION] as? Bool
        let uniqueId = hyperKycConfigArgument[UNIQUE_ID] as? String
        let metadata = hyperKycConfigArgument[METADATA] as? [String: String]
        
        if let accessToken = accessToken {
            let hyperKycConfig = HyperKycConfig(
                accessToken: accessToken,
                workflowId: workflowId,
                transactionId: transactionId
            )
            hyperKycConfig.setInputs(inputs: inputs ?? [String: String]())
            if let defaultLangCode = defaultLangCode {
                hyperKycConfig.setDefaultLangCode(language: defaultLangCode)
            }
            if let useLocation = useLocation {
                hyperKycConfig.setUseLocation(shouldUse: useLocation)
            }
            if let uniqueId = uniqueId {
                hyperKycConfig.setUniqueId(uuid: uniqueId)
            }
            if let metadata = metadata {
                hyperKycConfig.addMetadata(metadata: metadata)
            }
            HyperKyc.launch(
                context,
                hyperKycConfig: hyperKycConfig,
                callback
            )
        } else {
            let hyperKycConfig = HyperKycConfig(
                appId: appId!,
                appKey: appKey!,
                workflowId: workflowId,
                transactionId: transactionId
            )
            hyperKycConfig.setInputs(inputs: inputs ?? [String: String]())
            if let defaultLangCode = defaultLangCode {
                hyperKycConfig.setDefaultLangCode(language: defaultLangCode)
            }
            if let useLocation = useLocation {
                hyperKycConfig.setUseLocation(shouldUse: useLocation)
            }
            if let uniqueId = uniqueId {
                hyperKycConfig.setUniqueId(uuid: uniqueId)
            }
            if let metadata = metadata {
                hyperKycConfig.addMetadata(metadata: metadata)
            }
            HyperKyc.launch(
                context,
                hyperKycConfig: hyperKycConfig,
                callback
            )
        }
    }
}
