import Flutter
import UIKit
import HyperKYC

public class SwiftHyperkycFlutterPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let hyperKycChannel = FlutterMethodChannel(name: FLHyperKycConstants.HYPERKYC_CHANNEL_NAME, binaryMessenger: registrar.messenger())
        
        hyperKycChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result:@escaping FlutterResult) -> Void in
            if (call.method.caseInsensitiveCompare(FLHyperKycConstants.CREATE_UNIQUE_ID) == .orderedSame) {
                //  MARK: Create Unique ID
                let uniqueId = FLHyperKyc.createUniqueId()
                //  MARK: Pass uniqueId to Flutter plugin
                result(uniqueId)
            } else if (call.method.caseInsensitiveCompare(FLHyperKycConstants.LAUNCH) == .orderedSame) {
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return
                }
                let hyperKycConfigArgument = args[FLHyperKycConstants.HYPERKYCCONFIG] as! [String : Any]
                let completionHandler: (_ hyperKycResult: HyperKycResult) -> Void = {hyperKycResult in
                    
                    //  MARK: Process HyperKycResult and obtain final resultMap
                    let resultMap = processHyperKycResult(hyperKycResult: hyperKycResult)
                    
                    //  MARK: Pass the final resultMap to Flutter plugin
                    result(resultMap)
                }
                
                //  MARK: Call launch() function
                if let topController = UIApplication.shared.keyWindow?.rootViewController{
                    FLHyperKyc.launch(context: topController, hyperKycConfigArgument: hyperKycConfigArgument, callback: completionHandler)
                }
            } else {
                result(FlutterMethodNotImplemented)
            }
        })
    }
}

func processHyperKycResult(hyperKycResult: HyperKycResult) -> [String: Any] {
    //  MARK: Create the final result map which will include status, transactionId, details, errorCode, errorMessage, latestModule
    let resultMap = [
        FLHyperKycConstants.STATUS : hyperKycResult.status as String,
        FLHyperKycConstants.TRANSACTION_ID : hyperKycResult.transactionId as String,
        FLHyperKycConstants.DETAILS : hyperKycResult.details,
        FLHyperKycConstants.ERROR_CODE : hyperKycResult.errorCode ?? -1000,
        FLHyperKycConstants.ERROR_MESSAGE : hyperKycResult.errorMessage as String?,
        FLHyperKycConstants.LATEST_MODULE : hyperKycResult.latestModule as String?,
        FLHyperKycConstants.RAW_DATA_JSON_STRING : hyperKycResult.getRawDataJsonString() as String?
    ] as [String : Any]
    
    return resultMap
}
